from fastapi import FastAPI
import paramiko
from typing import Union
import logging
from pydantic import BaseModel
from fastapi.middleware.cors import CORSMiddleware
#import subprocess
# import json
# import os
# Configure the logging
logging.basicConfig(
    filename='dev.log',  # Specify the log file
    filemode='w',  # Append the log messages to the file
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    level=logging.DEBUG  # Set the logging level to DEBUG
)
# Create a logger
sysapp = logging.getLogger('dev')

class Item(BaseModel):
    cmd: str
    host: str
    username: str = 'epm'
    password: str = 'Wang%2023'
    portNum: str  = '22'

def runCommand(cmd, host, username='epm', password='Wang%2023', portNum=22):
    """
    Description: This function Execute commands remotely
    Parameters : cmd      - contains command to execute on remote host(STRING)
                 host     - hostname(IP) of remote host(STRING)
                 username - username of remote host(STRING)
                 password - password of remote host(STRING)
                 portNum  - port number of remote host(INT)(OPTIONAL)
                            default = 22
    """
    client = None
    try:
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        sysapp.info("Connecting to host: %s", host)
        # Connects to the remote host with provided credentials
        if password != None:
            client.connect(hostname=host, username=username, password=password, port=portNum)
        else:
            client.connect(hostname=host, key_filename='/home/epm/.ssh/id_rsa', port=portNum)
        sysapp.info("Connection established with host: %s and user: %s", host, username)

        sysapp.info("Executing command: %s", cmd)
        stdin, stdout, stderr = client.exec_command(cmd)
        stdin.close()
        outputBuffer = stdout.read()
        sysapp.info("Output Stream:\n%s", outputBuffer)
        errorBufffer = stderr.read()
        sysapp.info("Error Stream:\n%s", errorBufffer)

        return outputBuffer, errorBufffer
    except Exception:
        sysapp.exception("Failed to run command '%s' on remote host %s",
                          cmd, host)
        raise
    # Dis-connects the connection of remote host
    finally:
        if client:
            client.close()

app = FastAPI()

@app.put("/runSshCommand")
def runSshCommand(sshOps: Item):
    try:
        sysapp.info("Received command: %s", sshOps.model_dump_json())
        stdout, stderr = runCommand(sshOps.cmd, sshOps.host)
        sysapp.info("stdout %s",stdout )
        sysapp.info("stderr %s", stderr)
        return stdout
    except Exception as e:
        sysapp.error(e)
        return {"error": str(e)}
